require "twitter_client_app/version"
require "twitter_client_app/cli"

require 'sinatra'
require 'sinatra/json'
require 'sinatra/reloader'
require 'oauth'
require 'twitter'

module TwitterClientApp

  class App < Sinatra::Base
    set    :server, 'webrick'
    enable :sessions

    get '/' do
      send_file File.join(settings.public_folder, 'index.html')
    end

    get '/connect' do
      client = session[:access_token]
      json connect: !client.nil?
    end

    get '/login' do
      callback_url = "http://127.0.0.1:4567/callback"
      request_token = oauth_consumer.get_request_token(:oauth_callback => callback_url)

      session[:request_token] = request_token.token
      session[:request_token_secret] = request_token.secret
      redirect request_token.authorize_url
    end

    get '/callback' do
      request_token = OAuth::RequestToken.new(oauth_consumer, session[:request_token], session[:request_token_secret])

      access_token = nil
      begin
        access_token = request_token.get_access_token(
          {},
          oauth_token:  params[:oauth_token],
          oauth_verifier: params[:oauth_verifier])
      rescue OAuth::Unauthorized => @exception
        raise
      end

      # binding.pry

      session[:access_token] = access_token.token
      session[:access_token_secret] = access_token.secret

      redirect '/';
    end

    get '/logout' do
      session.clear
      redirect '/';
    end

    get '/tweets' do
      client = get_client(session[:access_token], session[:access_token_secret])

      result = client.search(HASH_TAG).take(SEARCH_LIMIT)
      data = result.map do |tweet|
        {
            id: tweet.id,
            userName: tweet.user.name,
            profile: tweet.user.profile_image_url.to_s,
            text: tweet.text
          }
      end

      data.to_json
    end

    post '/post' do
      client = get_client(session[:access_token], session[:access_token_secret])
      text = params['text']
      client.update(text + ' ' + HASH_TAG)
      json success: true
    end

    def self.setRoot(root_directory)
        set :root, root_directory
    end

    def oauth_consumer
      return OAuth::Consumer.new(CONSUMER_KEY, CONSUMER_SECRET, :site => "https://api.twitter.com")
    end

    def get_client(access_token, access_token_secret)
      return Twitter::REST::Client.new do |config|
        config.consumer_key        = CONSUMER_KEY
        config.consumer_secret     = CONSUMER_SECRET
        config.access_token        = access_token
        config.access_token_secret = access_token_secret
      end
    end

  end

end
